"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CliParser = void 0;
const chalk = require("chalk");
const commander_1 = require("commander");
const os_1 = require("os");
const process_1 = require("process");
const common_1 = require("../../../core/common");
const socket_1 = require("../../../core/socket");
class CliParser {
    constructor(version) {
        this.version = version;
    }
    parse(argv) {
        let config;
        const connectionTimeout = 5000;
        const reconnectionInterval = 10000;
        const description = `${chalk.bold('iola')} - a socket client with REST API`;
        const binaryEncodingChoices = this.choices(common_1.EnumUtil.values(common_1.BinaryEncoding));
        const ioTransportChoices = this.choices(common_1.EnumUtil.values(common_1.SocketIOTransport));
        const api = `API:${os_1.EOL}` +
            `  GET  /messages            Get message list${os_1.EOL}` +
            `  GET  /messages/{id}       Get message by id${os_1.EOL}` +
            `  POST /messages            Send message ${os_1.EOL}` +
            '  GET  /swagger             Get swagger';
        const websocketExamples = `Examples: ${os_1.EOL}` +
            `  $ iola ws ws://127.0.0.1:8080 ${os_1.EOL}` +
            `  $ iola ws ws://127.0.0.1:8080/?token=secret ${os_1.EOL}` +
            `  $ iola ws ws://127.0.0.1:8080 --header authorization:"Bearer token"${os_1.EOL}` +
            `  $ iola ws ws://127.0.0.1:8080 --binary-encoding utf8 ${os_1.EOL}` +
            '  $ iola ws ws://127.0.0.1:8080 --reply-timeout 3000 --no-emoji';
        const socketIOExamples = `Examples: ${os_1.EOL}` +
            `  $ iola io http://127.0.0.1:8080 ${os_1.EOL}` +
            `  $ iola io http://127.0.0.1:8080/?token=secret --transport websocket${os_1.EOL}` +
            `  $ iola io http://127.0.0.1:8080 --header authorization:"Bearer token"${os_1.EOL}` +
            `  $ iola io http://127.0.0.1:8080 --auth user:iola --auth pass:qwerty1${os_1.EOL}` +
            `  $ iola io http://127.0.0.1:8080 --binary-encoding utf8 ${os_1.EOL}` +
            '  $ iola io http://127.0.0.1:8080 --reply-timeout 3000 --no-emoji';
        const tcpExamples = `Examples: ${os_1.EOL}` +
            `  $ iola tcp 127.0.0.1:8080 ${os_1.EOL}` +
            `  $ iola tcp 127.0.0.1:8080 --sync ${os_1.EOL}` +
            `  $ iola tcp 127.0.0.1:8080 --binary-encoding utf8 ${os_1.EOL}` +
            '  $ iola tcp 127.0.0.1:8080 --no-emoji';
        const unixExamples = `Examples: ${os_1.EOL}` +
            `  $ iola unix ./unix.sock ${os_1.EOL}` +
            `  $ iola unix ./unix.sock --sync ${os_1.EOL}` +
            `  $ iola unix ./unix.sock --binary-encoding utf8 ${os_1.EOL}` +
            '  $ iola unix ./unix.sock --no-emoji';
        const program = (0, commander_1.createCommand)();
        program
            .name('iola')
            .version(this.version, '--version', 'Display version')
            .helpOption('--help', 'Display help')
            .addHelpText('before', os_1.EOL + description + os_1.EOL)
            .addHelpText('after', os_1.EOL + api + os_1.EOL)
            .addHelpCommand('help [command]', 'Display help for command');
        program
            .command('ws <address>')
            .description('Run websocket client')
            .enablePositionalOptions(false)
            .option('--api-port <port>', 'Set api port', '3000')
            .option('--api-host <host>', 'Set api host', '127.0.0.1')
            .option('--header <key:value...>', 'Set http headers')
            .option('--reply-timeout <timeout>', 'Set reply timeout in ms', '1000')
            .option('--binary-encoding <encoding>', `Set binary encoding ${binaryEncodingChoices}`)
            .option('--no-emoji', 'Disable emoji')
            .helpOption('--help', 'Display help')
            .addHelpText('before', ' ')
            .addHelpText('after', os_1.EOL + websocketExamples + os_1.EOL)
            .action((address, options) => {
            config = {
                socketType: socket_1.SocketType.WebSocket,
                socketAddress: address,
                apiPort: Number(options.apiPort),
                apiHost: options.apiHost,
                headers: this.parseKeyValueOption('-h, --header <key:value...>', options.header),
                binaryEncoding: options.binaryEncoding,
                emoji: options.emoji,
                replyTimeout: Number(options.replyTimeout),
                connectionTimeout: connectionTimeout,
                reconnectionInterval: reconnectionInterval,
            };
        });
        program
            .command('io <address>')
            .description('Run socket.io client')
            .enablePositionalOptions(false)
            .option('--api-port <port>', 'Set api port', '3000')
            .option('--api-host <host>', 'Set api host', '127.0.0.1')
            .option('--header <key:value...>', 'Set http headers')
            .option('--auth <key:value...>', 'Set authentication payload')
            .option('--transport <transport>', `Set transport ${ioTransportChoices}`)
            .option('--reply-timeout <timeout>', 'Set reply timeout in ms', '1000')
            .option('--binary-encoding <encoding>', `Set binary encoding ${binaryEncodingChoices}`)
            .option('--no-emoji', 'Disable emoji')
            .helpOption('--help', 'Display help')
            .addHelpText('before', ' ')
            .addHelpText('after', os_1.EOL + socketIOExamples + os_1.EOL)
            .action((address, options) => {
            config = {
                socketType: socket_1.SocketType.SocketIO,
                socketAddress: address,
                apiPort: Number(options.apiPort),
                apiHost: options.apiHost,
                headers: this.parseKeyValueOption('-h, --header <key:value...>', options.header),
                binaryEncoding: options.binaryEncoding,
                emoji: options.emoji,
                replyTimeout: Number(options.replyTimeout),
                ioAuth: this.parseKeyValueOption('-a, --auth <key:value...>', options.auth),
                ioTransport: options.transport,
                connectionTimeout: connectionTimeout,
                reconnectionInterval: reconnectionInterval,
            };
        });
        program
            .command('tcp <address>')
            .description('Run tcp client')
            .enablePositionalOptions(false)
            .option('--api-port <port>', 'Set api port', '3000')
            .option('--api-host <host>', 'Set api host', '127.0.0.1')
            .option('--sync', 'Enable sync mode')
            .option('--reply-timeout <timeout>', 'Set reply timeout in ms (sync mode only)', '1000')
            .option('--binary-encoding <encoding>', `Set binary encoding ${binaryEncodingChoices}`)
            .option('--no-emoji', 'Disable emoji')
            .helpOption('--help', 'Display help')
            .addHelpText('before', ' ')
            .addHelpText('after', os_1.EOL + tcpExamples + os_1.EOL)
            .action((address, options) => {
            config = {
                socketType: socket_1.SocketType.Tcp,
                socketAddress: address,
                apiPort: Number(options.apiPort),
                apiHost: options.apiHost,
                binaryEncoding: options.binaryEncoding,
                emoji: options.emoji,
                replyTimeout: Number(options.replyTimeout),
                netSync: options.sync,
                connectionTimeout: connectionTimeout,
                reconnectionInterval: reconnectionInterval,
            };
        });
        program
            .command('unix <address>')
            .description('Run unix client')
            .enablePositionalOptions(false)
            .option('--api-port <port>', 'Set api port', '3000')
            .option('--api-host <host>', 'Set api host', '127.0.0.1')
            .option('--sync', 'Enable sync mode')
            .option('--reply-timeout <timeout>', 'Set reply timeout in ms (sync mode only)', '1000')
            .option('--binary-encoding <encoding>', `Set binary encoding ${binaryEncodingChoices}`)
            .option('--no-emoji', 'Disable emoji')
            .helpOption('--help', 'Display help')
            .addHelpText('before', ' ')
            .addHelpText('after', os_1.EOL + unixExamples + os_1.EOL)
            .action((address, options) => {
            config = {
                socketType: socket_1.SocketType.Unix,
                socketAddress: address,
                apiPort: Number(options.apiPort),
                apiHost: options.apiHost,
                binaryEncoding: options.binaryEncoding,
                emoji: options.emoji,
                replyTimeout: Number(options.replyTimeout),
                netSync: options.sync,
                connectionTimeout: connectionTimeout,
                reconnectionInterval: reconnectionInterval,
            };
        });
        const opts = argv ? { from: 'user' } : undefined;
        program.parse(argv, opts);
        if (!config) {
            console.error('error: unknown command');
            process.exit(1);
        }
        return this.validateConfig(config);
    }
    validateConfig(config) {
        const errors = [];
        if (process_1.platform === 'win32' && config.socketType === socket_1.SocketType.Unix) {
            errors.push('unix-socket client does not work in windows');
        }
        if (isNaN(config.apiPort) || config.apiPort >= 65536 || config.apiPort < 0) {
            errors.push('api-port must be >= 0 and < 65536');
        }
        if (isNaN(config.replyTimeout) || config.replyTimeout < 1) {
            errors.push('reply-timeout must be a positive number');
        }
        if (errors.length > 0) {
            if (errors.length === 1) {
                console.error(`error: ${errors[0]}`);
                process.exit(1);
            }
            console.error(`errors:${os_1.EOL}` + errors.map(e => `  ${e}`).join(os_1.EOL));
            process.exit(1);
        }
        return config;
    }
    choices(choices) {
        const joined = choices
            .map(choice => `"${choice}"`)
            .join(',');
        return `(choices: ${joined})`;
    }
    parseKeyValueOption(optionName, optionValue) {
        if (optionValue) {
            const args = optionValue.map(item => item.split(':'));
            const isFormatValid = args.every(item => item.length > 1);
            if (!isFormatValid) {
                console.error(`error: option ${optionName} incorrect argument`);
                process.exit(1);
            }
            const parsed = {};
            args.forEach(arg => {
                const key = arg[0];
                const rawValue = arg.slice(1).join(':');
                const numericValue = Number(rawValue);
                let parsedValue = rawValue;
                if (!isNaN(numericValue) && rawValue !== '') {
                    parsedValue = numericValue;
                }
                if (['true', 'false'].includes(rawValue)) {
                    parsedValue = Boolean(rawValue);
                }
                if (rawValue === 'null') {
                    parsedValue = null;
                }
                parsed[key] = parsedValue;
            });
            return parsed;
        }
        return undefined;
    }
}
exports.CliParser = CliParser;
