"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetSocketClient = void 0;
const net_1 = require("net");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const common_1 = require("../../common");
const socket_1 = require("../../socket");
class NetSocketClient {
    constructor(options, eventStore, binaryMessageStore) {
        this._info = {
            type: options.type,
            address: options.address,
            originalAddress: options.address,
            connected: false,
            connecting: false,
        };
        this._options = options;
        this._eventStore = eventStore;
        this._binaryStore = binaryMessageStore;
    }
    get info() {
        return Object.assign({}, this._info);
    }
    get store() {
        return this._eventStore;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._info.connected) {
                this.clear();
                this._client = (0, net_1.createConnection)(this.netOptions());
                this._client.on('data', chunk => this._binaryStore.add(chunk));
                this._binaryStore.listen().subscribe(message => {
                    const unpacked = common_1.MessageUtil.unpack(message);
                    const encoding = this._options.binaryEncoding;
                    const eventMessage = {
                        format: unpacked.format,
                        size: unpacked.data.length,
                        data: unpacked.data,
                    };
                    if (encoding) {
                        eventMessage[encoding] = (Buffer.from(unpacked.data)).toString(encoding);
                    }
                    this._eventStore.add({
                        type: socket_1.SocketEventType.ReceivedMessage,
                        date: new Date(),
                        message: eventMessage,
                    });
                });
                this._client.on('error', err => this._eventStore.add({
                    type: socket_1.SocketEventType.Error,
                    date: new Date(),
                    message: {
                        message: err.message
                    },
                }));
                this._client.on('close', () => {
                    if (this._info.connected) {
                        this._eventStore.add({
                            type: socket_1.SocketEventType.Closed,
                            date: new Date(),
                            message: { code: 1, reason: '' },
                        });
                        this.clear();
                        this.retryConnection();
                    }
                });
                try {
                    const connect$ = (0, rxjs_1.fromEvent)(this._client, 'connect').pipe(common_1.RxJSUtil.timeout(this._options.connectionTimeout, `connection to ${this.info.address} is timed out`), (0, operators_1.tap)(() => this._info.connected = true), (0, operators_1.tap)(() => this._eventStore.add({
                        type: socket_1.SocketEventType.Connected,
                        date: new Date(),
                        message: this._info,
                    })), (0, operators_1.map)(() => undefined));
                    this._info.connecting = true;
                    yield (0, rxjs_1.firstValueFrom)(connect$);
                    this._info.connecting = false;
                    this._info.connected = true;
                }
                catch (error) {
                    this.clear();
                    throw error;
                }
            }
        });
    }
    sendData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const encoding = this._options.binaryEncoding;
            const packedString = common_1.MessageUtil.packToString(data);
            const packed = common_1.MessageUtil.packToBuffer(packedString.data);
            const eventMessage = {
                format: packed.format,
                size: packed.data.length,
                data: Array.from(packed.data),
            };
            if (encoding) {
                eventMessage[encoding] = packed.data.toString(encoding);
            }
            return this.send(Buffer.from(packed.data), eventMessage);
        });
    }
    sendBytes(bytes) {
        return __awaiter(this, void 0, void 0, function* () {
            const encoding = this._options.binaryEncoding;
            const packed = common_1.MessageUtil.packToBuffer(bytes);
            const eventMessage = {
                format: packed.format,
                size: bytes.length,
                data: bytes
            };
            if (encoding) {
                eventMessage[encoding] = packed.data.toString(encoding);
            }
            return this.send(packed.data, eventMessage);
        });
    }
    close() {
        var _a;
        this._binaryStore.close();
        (_a = this._client) === null || _a === void 0 ? void 0 : _a.destroy();
        this.clear();
    }
    clear() {
        var _a;
        (_a = this._client) === null || _a === void 0 ? void 0 : _a.removeAllListeners();
        this._binaryStore.clear();
        this._client = undefined;
        this._info.connected = false;
        this._info.connecting = false;
    }
    send(data, eventMessage) {
        const client = this._client;
        const connected = this._info.connected;
        if (!client || !connected) {
            throw new Error(`error: client is not connected to ${this.info.address}`);
        }
        return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
            client.write(data, err => {
                if (!err) {
                    const event = {
                        type: socket_1.SocketEventType.SentMessage,
                        date: new Date(),
                        message: eventMessage,
                    };
                    const messageId = this._eventStore.add(event);
                    resolve({ messageId });
                }
                reject(err);
            });
        }));
    }
    retryConnection() {
        const retryInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
            try {
                if (this._info.connected) {
                    clearInterval(retryInterval);
                }
                else if (!this._info.connecting) {
                    this._eventStore.add({
                        type: socket_1.SocketEventType.Reconnecting,
                        date: new Date(),
                        message: this._info,
                    });
                    yield this.connect();
                    clearInterval(retryInterval);
                }
            }
            catch (err) { }
        }), this._options.reconnectionInterval);
    }
    netOptions() {
        let netOpts = {};
        if (this._options.type === socket_1.SocketType.Tcp) {
            const parsedAddress = this._options.address.split(':');
            netOpts = {
                host: parsedAddress[0],
                port: parseInt(parsedAddress[1]),
            };
        }
        if (this._options.type === socket_1.SocketType.Unix) {
            netOpts = {
                path: this._options.address
            };
        }
        return netOpts;
    }
}
exports.NetSocketClient = NetSocketClient;
