"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketIOClient = void 0;
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const socket_io_client_1 = require("socket.io-client");
const common_1 = require("../../common");
const socket_1 = require("../../socket");
const url_util_1 = require("../../common/util/url.util");
class SocketIOClient {
    constructor(options, store) {
        this._info = {
            type: options.type,
            address: url_util_1.UrlUtil.removeSearchParams(options.address),
            originalAddress: options.address,
            connected: false,
            connecting: false,
        };
        this._options = options;
        this._store = store;
    }
    get store() {
        return this._store;
    }
    connect() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._info.connected) {
                this._client = (0, socket_io_client_1.io)(this._info.originalAddress, {
                    auth: this._options.ioAuth,
                    extraHeaders: this._options.headers,
                    rejectUnauthorized: false,
                    transports: this._options.ioTransport ? [this._options.ioTransport] : common_1.EnumUtil.values(common_1.SocketIOTransport)
                });
                this._client.onAny((event, ...args) => {
                    const message = args[0] === undefined
                        ? null
                        : args[0];
                    this.receiveMessage(event, message);
                });
                this._client.on('connect_error', err => {
                    if (this._info.connected) {
                        this._store.add({
                            type: socket_1.SocketEventType.Error,
                            date: new Date(),
                            message: {
                                message: err.message
                            },
                        });
                    }
                });
                this._client.on('disconnect', reason => {
                    if (this._info.connected) {
                        this._store.add({
                            type: socket_1.SocketEventType.Closed,
                            date: new Date(),
                            message: { reason },
                        });
                        this.clear();
                        this.retryConnection();
                    }
                });
                try {
                    const connect$ = (0, rxjs_1.fromEvent)(this._client, 'connect').pipe(common_1.RxJSUtil.timeout(this._options.connectionTimeout, `connection to ${this._info.address} is timed out`), (0, operators_1.tap)(() => this._info.connected = true), (0, operators_1.tap)(() => this._store.add({
                        type: socket_1.SocketEventType.Connected,
                        date: new Date(),
                        message: this._info,
                    })), (0, operators_1.map)(() => undefined));
                    this._info.connecting = true;
                    yield (0, rxjs_1.firstValueFrom)(connect$);
                    this._info.connecting = false;
                    this._info.connected = true;
                }
                catch (error) {
                    this.clear();
                    throw error;
                }
            }
        });
    }
    sendData(data, event = '*') {
        return __awaiter(this, void 0, void 0, function* () {
            const parsed = common_1.MessageUtil.parse(data);
            const eventMessage = {
                format: parsed.format,
                event: event,
                data: parsed.data
            };
            return this.emit(data, event, eventMessage);
        });
    }
    sendBytes(bytes, event = '*') {
        return __awaiter(this, void 0, void 0, function* () {
            const encoding = this._options.binaryEncoding;
            const packed = common_1.MessageUtil.packToBuffer(bytes);
            const eventMessage = {
                format: packed.format,
                size: bytes.length,
                event: event,
                data: bytes
            };
            if (encoding) {
                eventMessage[encoding] = packed.data.toString(encoding);
            }
            return this.emit(packed.data, event, eventMessage);
        });
    }
    close() {
        this.clear();
    }
    clear() {
        var _a, _b;
        (_a = this._client) === null || _a === void 0 ? void 0 : _a.removeAllListeners();
        (_b = this._client) === null || _b === void 0 ? void 0 : _b.close();
        this._client = undefined;
        this._info.connected = false;
        this._info.connecting = false;
    }
    retryConnection() {
        const retryInterval = setInterval(() => __awaiter(this, void 0, void 0, function* () {
            try {
                if (this._info.connected) {
                    clearInterval(retryInterval);
                }
                else if (!this._info.connecting) {
                    this._store.add({
                        type: socket_1.SocketEventType.Reconnecting,
                        date: new Date(),
                        message: this._info,
                    });
                    yield this.connect();
                    clearInterval(retryInterval);
                }
            }
            catch (err) { }
        }), this._options.reconnectionInterval);
    }
    emit(data, event, eventMessage) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = this._client;
            const connected = this._info.connected;
            if (!client || !connected) {
                throw new Error(`error: client is not connected to ${this._info.address}`);
            }
            const message = {
                type: socket_1.SocketEventType.SentMessage,
                date: new Date(),
                message: eventMessage,
            };
            let reply = {
                messageId: this._store.add(message)
            };
            const reply$ = (0, rxjs_1.from)(new Promise(resolve => {
                const resolveCb = (res) => {
                    reply.reply = this.receiveMessage(event, res);
                    resolve(reply);
                };
                client.emit(event, data, resolveCb);
            })).pipe((0, operators_1.timeout)(this._options.replyTimeout));
            try {
                reply = yield (0, rxjs_1.firstValueFrom)(reply$);
            }
            catch (err) { }
            return reply;
        });
    }
    receiveMessage(event, message) {
        const parsed = common_1.MessageUtil.parse(message);
        const encoding = this._options.binaryEncoding;
        let eventMessage = {
            format: parsed.format,
            event: event,
            data: parsed.data,
        };
        if (parsed.format === common_1.MessageFormat.ByteArray) {
            eventMessage = {
                format: parsed.format,
                size: parsed.data.length,
                event: event,
                data: parsed.data,
            };
            if (encoding) {
                eventMessage[encoding] = (Buffer.from(parsed.data)).toString(encoding);
            }
        }
        this._store.add({
            type: socket_1.SocketEventType.ReceivedMessage,
            date: new Date(),
            message: eventMessage,
        });
        return eventMessage;
    }
}
exports.SocketIOClient = SocketIOClient;
